// Generated by CoffeeScript 1.9.1
var each, execute, ldap, misc, string, wrap;

module.exports = function(options, callback) {
  return wrap(arguments, function(options, callback) {
    var do_diff, do_end, do_get_indexes, do_getdn, do_save, modified;
    modified = false;
    do_getdn = function() {
      if (options.hdb_dn) {
        return do_get_indexes();
      }
      if (typeof options.log === "function") {
        options.log("mecano `ldap_index`: get DN of the HDB to modify");
      }
      return execute({
        cmd: "ldapsearch -Y EXTERNAL -H ldapi:/// -b cn=config \"(olcSuffix= " + options.suffix + ")\" dn 2>/dev/null | egrep '^dn' | sed -e 's/^dn:\\s*olcDatabase=\\(.*\\)$/\\1/g'",
        ssh: options.ssh,
        log: options.log,
        stdout: options.stdout,
        stderr: options.stderr
      }, function(err, _, hdb_dn) {
        if (err) {
          return callback(err);
        }
        options.hdb_dn = hdb_dn.trim();
        return do_get_indexes();
      });
    };
    do_get_indexes = function() {
      if (typeof options.log === "function") {
        options.log("mecano `ldap_index`: list all indexes of the directory");
      }
      return execute({
        cmd: "ldapsearch -Y EXTERNAL -H ldapi:/// -b olcDatabase=" + options.hdb_dn + " \"(olcDbIndex=*)\" olcDbIndex",
        ssh: options.ssh,
        log: options.log,
        stdout: options.stdout,
        stderr: options.stderr
      }, function(err, _, stdout) {
        var attrlist, i, indexes, indices, len, line, match, ref;
        if (err) {
          return callback(err);
        }
        indexes = {};
        ref = string.lines(stdout);
        for (i = 0, len = ref.length; i < len; i++) {
          line = ref[i];
          if (!(match = /^olcDbIndex:\s+(.*)\s+(.*)/.exec(line))) {
            continue;
          }
          _ = match[0], attrlist = match[1], indices = match[2];
          indexes[attrlist] = indices;
        }
        return do_diff(indexes);
      });
    };
    do_diff = function(orgp) {
      var add, k, modify, ref, v;
      add = {};
      modify = {};
      ref = options.indexes;
      for (k in ref) {
        v = ref[k];
        if (orgp[k] == null) {
          add[k] = v;
        } else if (v !== orgp[k]) {
          modify[k] = [v, orgp[k]];
        }
      }
      if (Object.keys(add).length || Object.keys(modify).length) {
        return do_save(add, modify);
      } else {
        return do_end();
      }
    };
    do_save = function(add, modify) {
      var cmd, k, v;
      cmd = [];
      for (k in add) {
        v = add[k];
        cmd.push("add: olcDbIndex\nolcDbIndex: " + k + " " + v);
      }
      for (k in modify) {
        v = modify[k];
        cmd.push("delete: olcDbIndex\nolcDbIndex: " + k + " " + v[1] + "\n-\nadd: olcDbIndex\nolcDbIndex: " + k + " " + v[0]);
      }
      return execute({
        cmd: "ldapmodify -Y EXTERNAL -H ldapi:/// <<-EOF\ndn: olcDatabase=" + options.hdb_dn + "\nchangetype: modify\n" + (cmd.join('\n-\n')) + "\nEOF",
        ssh: options.ssh,
        log: options.log,
        stdout: options.stdout,
        stderr: options.stderr
      }, function(err, _, stdout) {
        if (err) {
          return callback(err);
        }
        modified = true;
        return do_end();
      });
    };
    do_end = function(err) {
      return callback(err, modified);
    };
    return do_getdn();
  });
};

each = require('each');

ldap = require('ldapjs');

execute = require('./execute');

misc = require('./misc');

string = require('./misc/string');

wrap = require('./misc/wrap');
